function [ estimated_nemo_data ] = estimate_best_guess( p_nemo_data )
%ESTIMATE_BEST_GUESS_POSITION Gets the most trusted position and stores it into SECOND_ORDER_INFORMATION struct.
%   WARNING: Do not use this function alone! If you want to have consistent
%   position data, please use
%   nemo.postprocessing.create_best_guess_position().
%
%   Creates a struct called best_guess_position in every profile and copies
%   either gps or iridium coordinates to it. If the status of the profile
%   is "ICE DETECTION", NaN will be placed into the member of the struct.
%   Same is being done, when there is neither gps nor iridium available.
%
%   Returns: Input struct, extended by best_guess_position member in SECOND_ORDER_INFORMATION.
%
%   Parameters:
%   p_nemo_data:    Either has to be a multidimensional struct (created by
%                   nemo.load.data()) or a string, where the nemo float data
%                   is stored.

%% Initialize return variables
estimated_nemo_data = false;

%% Parameter check
parameter_error = false; % default value if function fails

if (ischar(p_nemo_data)) % load profile data from file
    if (~isdir(p_nemo_data)) % parameter is not a directory
        parameter_error = true;
    else % load data
        estimated_nemo_data = nemo.load.data(p_nemo_data);
    end
else if (~iscell(p_nemo_data)) % raise error
        parameter_error = true;
    else
        estimated_nemo_data = p_nemo_data; % store nemo data in return variable
    end
end

if (parameter_error) % display error message
    disp([ mfilename ': Wrong parameter! Please provide either a ' ...
        'directory path or a struct created by nemo.load.data.m!']);
    return;
else % do some cleanup
    clear parameter_error;
end

%% Estimate best position
% Priority:
% 1) GPS
% 2) Iridium

% setup format structure
section_name = 'SECOND_ORDER_INFORMATION';
gps_section_name = 'SURFACE_GPS_DATA';
gps_section_format_name = 'SURFACE_GPS_DATA_FORMAT';
iridium_section_name = 'IRIDIUM_DATA';
iridium_section_format_name = 'IRIDIUM_DATA_FORMAT';

% process every single float
for o_float_index = 1:length(estimated_nemo_data)
    % get profile count of current float
    profile_count = length(estimated_nemo_data{o_float_index});
    
    % run through every profile
    for o_profile_index = 1:profile_count
        % get current profile
        current_profile = ...
            estimated_nemo_data{o_float_index}{o_profile_index};
        if o_profile_index ~= 1
            % get previous profile
            previous_profile = ...
                estimated_nemo_data{o_float_index}{o_profile_index - 1};
        else
            previous_profile = {};
        end
        
        if o_profile_index < profile_count
            % get previous profile
            next_profile = ...
                estimated_nemo_data{o_float_index}{o_profile_index + 1};
        else
            next_profile = {};
        end
        
        % check status first
        if (nemo.profile.has_status(current_profile, 'ICE DETECTION') ...
                || (~isempty(previous_profile) && nemo.profile.has_status(previous_profile, 'ICE DETECTION')))
            % no position is available
            current_profile.(section_name).best_guess_position_lat = NaN;
            current_profile.(section_name).best_guess_position_lon = NaN;
            current_profile.(section_name).best_guess_position_src = NaN;
        elseif (nemo.profile.has_gps(current_profile))
            % gps available so take it
            % get indices of lat and lon
            index_lat = find(not(cellfun('isempty', ...
                strfind(current_profile.(gps_section_format_name), 'lat') ...
                )));
            index_lon = find(not(cellfun('isempty', ...
                strfind(current_profile.(gps_section_format_name), 'lon') ...
                )));
            current_profile.(section_name).best_guess_position_lat = ...
                current_profile.(gps_section_name)(index_lat);
            current_profile.(section_name).best_guess_position_lon = ...
                current_profile.(gps_section_name)(index_lon);
            current_profile.(section_name).best_guess_position_src = ...
                'gps';
            
            clear index_lon index_lat;
        elseif (nemo.profile.has_no_gps_but_iridium(current_profile) ...
                && (isempty(next_profile) || nemo.profile.verify_iridium(current_profile, next_profile)) ...
                )
            % iridium available
            % get indices of lat and lon
            index_lat = find(not(cellfun('isempty', ...
                strfind(current_profile.(iridium_section_format_name), 'mean_lat') ...
                )));
            index_lon = find(not(cellfun('isempty', ...
                strfind(current_profile.(iridium_section_format_name), 'mean_lon') ...
                )));
            current_profile.(section_name).best_guess_position_lat = ...
                current_profile.(iridium_section_name)(index_lat);
            current_profile.(section_name).best_guess_position_lon = ...
                current_profile.(iridium_section_name)(index_lon);
            current_profile.(section_name).best_guess_position_src = ...
                'iridium';
            clear index_lon index_lat;
        else
            current_profile.(section_name).best_guess_position_lat = NaN;
            current_profile.(section_name).best_guess_position_lon = NaN;
            current_profile.(section_name).best_guess_position_src = NaN;
        end
        
        % store updated profile
        estimated_nemo_data{o_float_index}{o_profile_index} = current_profile;
        
    end
    
    
end


end

