function [ rfb, metadata ] = create( p_nemo_float, p_float_metadata )
%CREATE Creates the rfb structure of the given floats.
%   
%   Parameters:
%   p_nemo_float:           The floats to process.
%   p_float_metadata:       The floats metadata.
%
%   Returns:
%   rfb:                    The rfb structure.
%   metadata:               The corresponding metadata. Floats without
%                           RAFOS have been sorted out.

%% Initialize return variables
rfb = false;

%% Parameter check
if (~iscell(p_nemo_float)) % display error message
    disp([ mfilename ': Wrong parameter! Please check your input parameters carefully!']);
    return;
end

% if it is the first profile, it is a struct, otherwise a cell
switch (class(p_nemo_float{1}))
    case 'cell'
        % sort out all floats without rafos data
%         sending_floats_metadata = {};
%         for o = 1:length(p_nemo_float) % run through every float
%             current_serial = p_nemo_float{o}{1}.FLOAT_IDENTIFICATION.float_serial_number;
%             for i = 1:length(p_float_metadata)
%                 if (p_float_metadata{i}.GENERAL_INFO.SERIAL == current_serial)
%                     sending_floats_metadata{o} = p_float_metadata{i};
%                 end
%             end
%         end
%         rfb = cellfun(@create_rfb_float, p_nemo_float, sending_floats_metadata', 'UniformOutput', false);
        rfb = cellfun(@create_rfb_float, p_nemo_float, p_float_metadata, 'UniformOutput', false);
%         metadata = sending_floats_metadata;
        metadata = p_float_metadata;
    case 'struct'
        rfb = create_rfb_float(p_nemo_float, p_float_metadata);
        metadata = p_float_metadata;
    otherwise
        disp([mfilename ': Type ' class(p_nemo_float{1}) ' not supported!' ]);
        return
end


    function rfb_float = create_rfb_float(float, metadata)
        
        if(~isfield(float{1}, 'SECOND_ORDER_INFORMATION'))
            disp([ mfilename ': Generation of rfb file requires field SECOND_ORDER_INFORMATION!']);
            return;
        end

        rfb_float = struct();
        
        %% Create header information
        FLOAT = struct( ...
            'floatname', float{1}.FLOAT_IDENTIFICATION.float_serial_number, ...
            'type', '', ...
            'projectname', metadata.RAFOS.GENERAL_INFO.projectname, ...
            'pttdec', '', ...
            'ptthex', '', ...
            'pttrep', metadata.RAFOS.GENERAL_INFO.pttrep, ...
            'launchpos', float{1}.DEPLOYMENT_INFO.deployment_position, ...
            'launchtime', [ ...
            fliplr(float{1}.DEPLOYMENT_INFO.deployment_date), ...
            float{1}.DEPLOYMENT_INFO.deployment_time ...
            ], ...
            'recoverpos', [ ...
            float{end}.SECOND_ORDER_INFORMATION.best_guess_position_lat, ...
            float{end}.SECOND_ORDER_INFORMATION.best_guess_position_lon ...
            ], ...
            'recovertime', float{end}.SECOND_ORDER_INFORMATION.best_guess_datevec, ...
            'offset', float{end}.SECOND_ORDER_INFORMATION.best_guess_datevec, ...
            'cycle', [ ...
            1, ...
            float{1}.DEPLOYMENT_INFO.deployment_position, ...
            fliplr(float{1}.DEPLOYMENT_INFO.deployment_date), ...
            float{1}.DEPLOYMENT_INFO.deployment_time, ...
            float{end}.SECOND_ORDER_INFORMATION.best_guess_position_lat, ...
            float{end}.SECOND_ORDER_INFORMATION.best_guess_position_lon, ...
            float{end}.SECOND_ORDER_INFORMATION.best_guess_datevec ...
            ], ...
            'phasespercycle', 0, ... % is being set later, not available up to this point
            'schedule', metadata.RAFOS.GENERAL_INFO.schedule, ...
            'phasereftime', metadata.RAFOS.GENERAL_INFO.phasereftime, ...
            'windowsperphase', metadata.RAFOS.GENERAL_INFO.windowsperphase, ...
            'toaperwindow', metadata.RAFOS.GENERAL_INFO.toaperwindow, ...
            'toaperphase', metadata.RAFOS.GENERAL_INFO.toaperphase, ...
            'correlationrange', metadata.RAFOS.GENERAL_INFO.correlationrange, ...
            'windowrange', metadata.RAFOS.GENERAL_INFO.windowrange, ...
            'windowstart', metadata.RAFOS.GENERAL_INFO.windowstart, ...
            'windowlength', metadata.RAFOS.GENERAL_INFO.windowlength, ...
            'progdepth', metadata.RAFOS.GENERAL_INFO.progdepth, ...
            'progtemp', metadata.RAFOS.GENERAL_INFO.progtemp, ...
            'tempequation', metadata.RAFOS.GENERAL_INFO.tempequation, ...
            'tempcoeff', metadata.RAFOS.GENERAL_INFO.tempcoeff, ...
            'presequation', metadata.RAFOS.GENERAL_INFO.presequation, ...
            'prescoeff', metadata.RAFOS.GENERAL_INFO.prescoeff, ...
            'signal_length', metadata.RAFOS.GENERAL_INFO.signal_length, ...
            'comment', metadata.RAFOS.GENERAL_INFO.comment, ...
            'edited_by', metadata.RAFOS.GENERAL_INFO.edited_by, ...
            'edited_on', datevec(now), ...
            'variables', metadata.RAFOS.GENERAL_INFO.variables ...
            );

        SAT_FORMAT = struct( ...
            'lat_sat', metadata.RAFOS.GPS_FORMAT.lat, ...
            'lon_sat', metadata.RAFOS.GPS_FORMAT.lon, ...
            'src', metadata.RAFOS.GPS_FORMAT.src, ...
            'year_sat', metadata.RAFOS.GPS_FORMAT.year, ...
            'month_sat', metadata.RAFOS.GPS_FORMAT.month, ...
            'day_sat', metadata.RAFOS.GPS_FORMAT.day, ...
            'hour_sat', metadata.RAFOS.GPS_FORMAT.hour, ...
            'minutes_sat', metadata.RAFOS.GPS_FORMAT.minutes, ...
            'seconds_sat', metadata.RAFOS.GPS_FORMAT.seconds, ...
            'year_rtc', metadata.RAFOS.GPS_FORMAT.year_rtc, ...
            'month_rtc', metadata.RAFOS.GPS_FORMAT.month_rtc, ...
            'day_rtc', metadata.RAFOS.GPS_FORMAT.day_rtc, ...
            'hour_rtc', metadata.RAFOS.GPS_FORMAT.hour_rtc', ...
            'minutes_rtc', metadata.RAFOS.GPS_FORMAT.minutes_rtc, ...
            'seconds_rtc', metadata.RAFOS.GPS_FORMAT.seconds_rtc ...
            );
        
        VARIABLE_LIST = struct( ...
            'line_number', metadata.RAFOS.VARIABLE_LIST.line_number, ...
            'start_listen_yr', metadata.RAFOS.VARIABLE_LIST.start_listen_yr, ...
            'start_listen_mon', metadata.RAFOS.VARIABLE_LIST.start_listen_mon, ...
            'start_listen_day', metadata.RAFOS.VARIABLE_LIST.start_listen_day, ...
            'start_listen_hr', metadata.RAFOS.VARIABLE_LIST.start_listen_hr, ...
            'start_listen_min', metadata.RAFOS.VARIABLE_LIST.start_listen_min, ...
            'start_listen_sec', metadata.RAFOS.VARIABLE_LIST.start_listen_sec, ...
            'time_of_arrival', metadata.RAFOS.VARIABLE_LIST.time_of_arrival, ...
            'correlation_height', metadata.RAFOS.VARIABLE_LIST.correlation_height, ...
            'pressure', metadata.RAFOS.VARIABLE_LIST.pressure, ...
            'temperature', metadata.RAFOS.VARIABLE_LIST.temperature, ...
            'pres_counts', metadata.RAFOS.VARIABLE_LIST.pres_counts, ...
            'temp_counts', metadata.RAFOS.VARIABLE_LIST.temp_counts ...
            );
        
        
        %% Collect all data information
        
        SAT_DATA = nemo.float.collect(float, 'SURFACE_GPS_DATA', '', 'combine');
        
        rafos_data = nemo.float.collect(float, 'RAFOS_VALUES', '', 'combine');
        if (isempty(rafos_data))
            rfb_float = false;
            return;
        end
        % apply some corrections
        hour_scheduled = 30;
        travel_correction = -60 * (rafos_data(:, 6) - hour_scheduled) ...
            + rafos_data(:, 7);
        % the following line throws an error in 2016a
        % rafos_data(:, 9:2:19) = rafos_data(:, 9:2:19) + travel_correction;
        rafos_data(:, 9) = rafos_data(:, 9) + travel_correction;
        rafos_data(:, 11) = rafos_data(:, 11) + travel_correction;
        rafos_data(:, 13) = rafos_data(:, 13) + travel_correction;
        rafos_data(:, 15) = rafos_data(:, 15) + travel_correction;
        rafos_data(:, 17) = rafos_data(:, 17) + travel_correction;
        rafos_data(:, 19) = rafos_data(:, 19) + travel_correction;
        % remove all unknown dates
        rafos_data = rafos_data(isfinite(datenum(rafos_data(:, 2:4))), :);
        
        % insert NaN for every missing row (compatibility fix for artoa)
        % time_axis = datenum( ...
        %     datestr(FLOAT.launchtime,'dd-mmm-yyyy')) ...
        %     :p_rfb_config.GENERAL_INFO.artoa_expected_cycle ...
        %     :datenum(datestr(FLOAT.recovertime,'dd-mmm-yyyy') ...
        %     - 1 ...
        %     );
        time_axis = [ ...
            datenum(fliplr(float{1}.DEPLOYMENT_INFO.deployment_date)) ...
            :metadata.RAFOS.GENERAL_INFO.artoa_expected_cycle ...
            :datenum(float{end}.SECOND_ORDER_INFORMATION.best_guess_datevec(1:3)) ...
            ];
        rafos_dates = datenum(rafos_data(:, 2:4));
        
        [~, id1] = intersect(time_axis, rafos_dates);
        [~, id2] = intersect(rafos_dates, time_axis);
        
        if (size(rafos_data, 1) > length(time_axis))
            disp([ mfilename ...
                ': More RAFOS values available than possible for float number ' ...
                num2str(float{1}.FLOAT_IDENTIFICATION.float_serial_number) '!' ...
                ]);
        end
        
        if (strcmp(float{end}.SECOND_ORDER_INFORMATION.best_guess_datevec_src, 'none'))
            % create resulting matrix
            RAFOS_VALUES = rafos_data;
            %RAFOS_VALUES(id1, 1) = id1;

            % change order of columns and remove unnecessary ones
            tmp = [ ...
                RAFOS_VALUES(:, 1:19), ...
                RAFOS_VALUES(:, 22), ...
                RAFOS_VALUES(:, 20) ...
                ];
            RAFOS_VALUES = tmp;
            clear tmp

            %% Set values that were not available until now
            FLOAT.phasespercycle = length(rafos_dates) + 1;
        else
            % create resulting matrix
            RAFOS_VALUES = nan(length(time_axis), size(rafos_data, 2));
            RAFOS_VALUES(id1, :) = rafos_data(id2, :);
            RAFOS_VALUES(id1, 1) = id1;

            % change order of columns and remove unnecessary ones
            tmp = [ ...
                RAFOS_VALUES(:, 1:19), ...
                RAFOS_VALUES(:, 22), ...
                RAFOS_VALUES(:, 20) ...
                ];
            RAFOS_VALUES = tmp;
            clear tmp

            %% Set values that were not available until now
            FLOAT.phasespercycle = length(time_axis) + 1;
        end
        
        %% Write resulting structure
        
        rfb_float.FLOAT = FLOAT;
        rfb_float.VARIABLE_LIST = VARIABLE_LIST;
        if ~isempty(SAT_DATA)
            rfb_float.SAT_FORMAT = SAT_FORMAT;
            rfb_float.SAT_DATA = SAT_DATA;
        end
        rfb_float.DATA = RAFOS_VALUES;
    end


end

